/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "kone_orig_macro.h"
#include "g_roccat_helper.h"
#include "roccat_helper.h"

static guint8 kone_original_macro_calc_checksum(KoneOriginalMacro const *macro) {
	return ROCCAT_BYTESUM_PARTIALLY(macro, KoneOriginalMacro, macro_name[0], checksum);
}

static void kone_original_macro_set_checksum(KoneOriginalMacro *macro) {
	macro->checksum = kone_original_macro_calc_checksum(macro);
}

static void kone_original_macro_set_macro_name(KoneOriginalMacro *macro, gchar const *new_name) {
	g_strlcpy((gchar *)(macro->macro_name), new_name, KONE_ORIGINAL_MACRO_NAME_LENGTH);
}

static void roccat_macro_keystroke_to_kone_original_macro_keystroke(RoccatMacroKeystroke const *from, KoneOriginalMacroKeystroke *to) {
	to->key = from->key;
	kone_original_macro_keystroke_set_period(to, roccat_macro_keystroke_get_period(from));
	if (from->action == ROCCAT_MACRO_KEYSTROKE_ACTION_PRESS)
		to->action = KONE_ORIGINAL_MACRO_KEYSTROKE_ACTION_PRESS;
	else
		to->action = KONE_ORIGINAL_MACRO_KEYSTROKE_ACTION_RELEASE;
}

static KoneOriginalMacro *roccat_macro_to_kone_original_macro(RoccatMacro const *roccat_macro) {
	KoneOriginalMacro *result;
	guint i, count;

	count = roccat_macro_keystrokes_get_count(&roccat_macro->keystrokes);
	if (count > KONE_ORIGINAL_MACRO_KEYSTROKES_NUM) {
		g_warning("Macro %s/%s is too long to convert", roccat_macro->macro_set, roccat_macro->macro);
		return NULL;
	}

	result = g_malloc0(sizeof(KoneOriginalMacro));
	kone_original_macro_set_macro_name(result, roccat_macro->macro);
	kone_original_macro_set_count(result, count);

	for (i = 0; i < count; ++i) {
		roccat_macro_keystroke_to_kone_original_macro_keystroke(&roccat_macro->keystrokes.keystrokes[i], &result->keystrokes[i]);
	}

	kone_original_macro_set_checksum(result);

	return result;
}

gboolean kone_original_macros_export(gchar const *filename, RoccatMacros *roccat_macros, GError **error) {
	GKeyFile *kone_original_macro_file = g_key_file_new();
	RoccatMacro *roccat_macro;
	KoneOriginalMacro *kone_original_macro;
	gchar *data, **groups, **group, **keys, **key;
	gsize length;
	guint macroset_counter = 0;
	guint macro_counter = 0;
	gchar *intkey;

	groups = roccat_macros_get_macro_sets(roccat_macros, NULL);
	for (group = groups; *group; ++group) {
		keys = roccat_macros_get_macros(roccat_macros, *group, NULL, error);
		if (*error)
			goto free_groups;

		intkey = g_strdup_printf("%i", macroset_counter);
		g_key_file_set_string(kone_original_macro_file, KONE_ORIGINAL_MACRO_MAIN_GROUP_NAME, intkey, *group);
		g_free(intkey);

		for (key = keys; *key; ++key) {
			roccat_macro = roccat_macros_get(roccat_macros, *group, *key, error);
			if (!roccat_macro)
				goto free_keys;

			kone_original_macro = roccat_macro_to_kone_original_macro(roccat_macro);
			roccat_macro_free(roccat_macro);
			if (!kone_original_macro)
				break;

			intkey = g_strdup_printf("%i", macro_counter);
			roccat_key_file_set_binary(kone_original_macro_file, *group, intkey, kone_original_macro, sizeof(KoneOriginalMacro));
			g_free(intkey);
			g_free(kone_original_macro);
			++macro_counter;
		}
		g_key_file_set_integer(kone_original_macro_file, *group, KONE_ORIGINAL_MACRO_COUNT_NAME, macro_counter);
		g_strfreev(keys);
		++macroset_counter;
		macro_counter = 0;
	}
	g_strfreev(groups);
	g_key_file_set_integer(kone_original_macro_file, KONE_ORIGINAL_MACRO_MAIN_GROUP_NAME, KONE_ORIGINAL_MACRO_COUNT_NAME, macroset_counter);

	data = g_key_file_to_data(kone_original_macro_file, &length, error);
	g_key_file_free(kone_original_macro_file);
	if (*error)
		goto exit;

	g_file_set_contents(filename, data, length, error);
	g_free(data);
	if (*error)
		goto exit;

	return TRUE;
free_keys:
	g_strfreev(keys);
free_groups:
	g_strfreev(groups);
	g_key_file_free(kone_original_macro_file);
exit:
	return FALSE;
}
